package Sophos::ValidatedFetch::Utils;

use strict;
use warnings;
use PPM::PureMessage::Utils qw(pmx_prefix);
use base qw(Exporter);
use Fcntl ':mode';

our @EXPORT = qw(openssl set_openssl_path cert_file set_cert_file openssl_path get_ua slurpit manifest_cache_location);

sub slurpit {
    my ($file) = @_;
    open(my $fh, $file) or die "Could not open file '$file': $!";
    my $contents = do { local $/; <$fh> };
    close $fh or die "Could not close file '$file': $!";
    return $contents;
}


sub manifest_cache_location {
    return pmx_prefix() . '/var/manifest_cache';
}

my $openssl_location;
my $ca_cert;

sub openssl {
  my ($cmd) = @_;
  if (!$openssl_location) {
      $openssl_location = openssl_path();
  }
  my $output = `$openssl_location $cmd`;
  return $output;
}

sub set_openssl_path {
  $openssl_location = shift;  
}

sub openssl_path {
  return $openssl_location if $openssl_location;

  if ($ENV{OPENSSL_PATH}) {
      return $ENV{OPENSSL_PATH};  
  } else {
      my $PREFIX = pmx_prefix();
      if (-e $PREFIX && -e "$PREFIX/bin/openssl") {
        return $PREFIX . "/bin/openssl";
      }
  }
  return;
}

my $WARN_TEST_CERT = 0;
sub warn_test_cert {
    my $location = shift;
    return if $WARN_TEST_CERT++;
    warn <<EOF;

*****************************************************************************************
*      USING TEST CERTIFICATE.  DATA WILL NOT BE VALIDATED AS ORIGINATING AT SOPHOS     *
       cert location: $location
*****************************************************************************************

EOF
}

my $IGNORE_TEST_CERT = 0;

sub emit_error_reason {
    my ($file,@perms) = @_;
    warn "Test ca cert file had permission problems: '$file' exists but is group writable.  Ignoring.\n" if $perms[2] & S_IWGRP;
    warn "Test ca cert file had permission problems: '$file' exists but is world writable.  Ignoring.\n" if $perms[2] & S_IWOTH;
    warn "Test ca cert file had permission problems: '$file' exists but is not owned by root.  Ignoring.\n" if $perms[4] != 0;
    warn "Test ca cert file had permission problems: '$file' exists but group is not root.  Ignoring.\n" if $perms[5] != 0;
}

sub cert_file {

    my $test_cert = "/etc/puremessage-test-rootca.crt";

    if (-e $test_cert && !$IGNORE_TEST_CERT) {

        my @perms = stat($test_cert);

        if ( ($perms[4] == 0) and ($perms[5] == 0) and not($perms[2] & S_IWGRP) and not($perms[2] & S_IWOTH)) {


                 warn_test_cert($test_cert);
                 $ca_cert = $test_cert;
                 return $ca_cert;


        } else {
            $IGNORE_TEST_CERT = 1;
            emit_error_reason($test_cert,@perms);
        }

    }

    die "No cert file defined. Validation cannot continue." unless $ca_cert;
    return $ca_cert;
}

sub set_cert_file {
    $ca_cert = shift;
}

1;
