package Sophos::ValidatedFetch::VerifyChain;

use strict;
use warnings;

use File::Path qw(mkpath rmtree);
use Sophos::ValidatedFetch::Utils qw(openssl cert_file);
use PPM::PureMessage::Utils qw(create_tmp_dir);

my $ca_file;

sub new {
  my ($class, %args) = @_;
  my $self = {};
  $self->{certs} = $args{certs};
  $ca_file = cert_file(); 
  die "Can't find certificate" unless $ca_file;
  return bless $self, $class;
}

sub valid {
  my ($self) = @_;

  my $tmp_dir = create_tmp_dir('sophos-validation',1);
  $tmp_dir .= '/' unless $tmp_dir =~ m{/$};
  $self->{certs_on_disk} = [];
  my $i = 0;
  for my $cert (@{ $self->{certs} }){
    my $filename = "${tmp_dir}cert_$i";
    open(my $fh, ">$filename") or die "Could not open file '$filename': $!";
    print {$fh} $cert;
    close $fh or die "Could not close file '$filename': $!";

    push @{ $self->{certs_on_disk} }, $filename;
    $i++;
  }

  my $certs_str = join ' ', reverse(@{ $self->{certs_on_disk} });

  my $output = openssl("verify -CAfile $ca_file -purpose smimesign -untrusted $certs_str 2>&1");
  unless( $output =~ m{cert_0: OK} ) {
     die "Manifest was not signed by a Sophos approved certificate\n" . ('-' x 80) . "\n$output\n";
  }

  return 1;
}

1;
