package Sophos::ValidatedFetch::VerifyManifest;

use PPM::PureMessage::Utils qw(create_tmp_dir);
use strict;
use warnings;

use File::Path qw(mkpath rmtree);


use Sophos::ValidatedFetch::VerifyChain;
use Sophos::ValidatedFetch::VerifySignature;

sub new {
    my ($class, %args) = @_;

    my $self = {};
    $self->{manifest} = $args{manifest};

    return bless $self, $class;
}

sub valid {
    my ($self) = @_;

    # Create temporary directory
    my $tmp_dir = create_tmp_dir('sophos-validation',1);

    my ($manifest, $signature, @certs) = $self->_extract_manifest_elements();

    # verify certificate chain
    my $chain_v = Sophos::ValidatedFetch::VerifyChain->new(certs => [@certs]);
    $chain_v->valid();

    # verify signature
    my $sig_v = Sophos::ValidatedFetch::VerifySignature->new(
      manifest => $manifest, signature => $signature, certificate => $certs[0]
    );
    $sig_v->valid();

    return 1;
}

sub _extract_manifest_elements {
    my ($self) = @_;

    my ($manifest, $signature, @certs);

    my $ref = \$manifest;
    for my $line (split /\n/, $self->{manifest}){
      $line .= "\n";
      if($line =~ m{-----BEGIN SIGNATURE-----}){
        $ref = \$signature;
        next;
      }

      if($line =~ m{-----END SIGNATURE-----}){
        next;
      }

      if($line =~ m{-----BEGIN CERTIFICATE-----}){
        push(@certs, '');
        $ref = \$certs[-1];
      }

      $$ref .= $line;
    }

    return ($manifest, $signature, @certs);
}

sub DESTROY {
  rmtree(create_tmp_dir('sophos-validation',1));
}

1;
