package Sophos::ValidatedFetch::VerifySignature;

use strict;
use warnings;

use File::Path qw(mkpath rmtree);
use Sophos::ValidatedFetch::Utils qw(cert_file openssl);
use PPM::PureMessage::Utils qw(create_tmp_dir);

my $ca_file;

sub new {
  my ($class, %args) = @_;
  my $self = {};
  $self->{cert} = $args{certificate};
  $self->{manifest} = $args{manifest};
  $self->{signature} = $args{signature};
  $ca_file = cert_file();
  die "Can't find certificate" unless $ca_file;
  return bless $self, $class;
}


sub valid {
  my ($self) = @_;

  my $tmp_dir = create_tmp_dir('sophos-validation',1);

  open my $man_fh, ">${tmp_dir}manifest" or die "Could not open file: $!";
  print { $man_fh } $self->{manifest};
  close $man_fh;

  open my $sig_fh, ">${tmp_dir}signature_tmp" or die "Could not open file: $!";
  print { $sig_fh } $self->{signature};
  close $sig_fh;

  openssl("base64 -in ${tmp_dir}signature_tmp -out ${tmp_dir}signature -d");
  unlink "${tmp_dir}signature_tmp";

  open my $cert_fh, ">${tmp_dir}certificate_tmp" or die "Could not open file: $!";
  print { $cert_fh } $self->{cert};
  close $cert_fh;
  
  openssl("x509 -pubkey -in ${tmp_dir}certificate_tmp > ${tmp_dir}certificate.pem");
  unlink "${tmp_dir}certificate_tmp";

  my $output = openssl("dgst -sha1 -verify ${tmp_dir}certificate.pem -signature ${tmp_dir}signature ${tmp_dir}manifest");

  unless( $output =~ /Verifi(cation|ed) OK/ ) {
     die "Manifest was not signed by Sophos\n" . ('-' x 80) . "\n$output";
  }

  return 1;
}

1;
